==> gau-0.3.1.tar.gz: gau-0.3.1/ansiinput.c <==
/* this file is a part of gau software, (C) Hwang chi-deok 1997, 1998         */

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <gdk/hangulinput.h>
#include "ansiterm.h"


void scroll_mode_key_press(AnsiTerm *term, GdkEventKey *ev);
int crlf = 0;
#if HANGUL
void flush_hangul(AnsiTerm *);
void push_hangul(AnsiTerm *, char *buf);
void correct_hangul(AnsiTerm *, char *buf);
#endif
void scroll_mode_page_up(AnsiTerm *);
void scroll_mode_page_down(AnsiTerm *);

#if HANGUL
void
hangul_input_toggle(gpointer foo)
{
	if (hangul_input_state_get()) {
		hangul_input_clear();
		hangul_input_state_set(FALSE);
	} else hangul_input_state_set(TRUE);
}
#endif

void
scroll_up(AnsiTerm *term)
{
	scroll_mode_page_up(term);
}

void
keymap_init(AnsiTerm *term)
{
	KeyMap *keymap;
#if HANGUL
	keymap = g_new(KeyMap, 1);
	keymap->keyval = GDK_space;
	keymap->state = GDK_SHIFT_MASK;
	keymap->func = hangul_input_toggle;
	term->keymap_list = g_list_append(term->keymap_list, keymap);
#endif
	keymap = g_new(KeyMap, 1);
	keymap->keyval = GDK_Page_Up;
	keymap->state = GDK_SHIFT_MASK;
	keymap->func = (KeyMapFunc)scroll_up;
	keymap->data = term;
	term->keymap_list = g_list_append(term->keymap_list, keymap);
}

/* Structure to hold escape sequences. */
struct escseq {
  int code;
  char *linux_con;
  char *vt100_app;
  char *ansi;
};

/* Escape sequences for different terminal types. */
static struct escseq vt_keys[] = {
  { GDK_F1,      "[[A", "[11~", "OP" },
  { GDK_F2,      "[[B", "[12~", "OQ" },
  { GDK_F3,      "[[C", "[13~", "OR" },
  { GDK_F4,      "[[D", "[14~", "OS" },
  { GDK_F5,      "[[E", "[15~", "OT" },
  { GDK_F6,      "[17~", "[17~", "OU" },
  { GDK_F7,      "[18~", "[18~", "OV" },
  { GDK_F8,      "[19~", "[19~", "OW" },
  { GDK_F9,      "[20~", "[20~", "OX" },
  { GDK_F10,     "[21~", "[21~", "OY" },
  { GDK_F11,     "[23~", "[23~", "OZ" },
  { GDK_F12,     "[24~", "[24~", "OA" },
  { GDK_Up,      "[A",   "OA",   "[A" },
  { GDK_Left,    "[D",   "OD",   "[D" },
  { GDK_Right,   "[C",   "OC",   "[C" },
  { GDK_Down,    "[B",   "OB",   "[B" },
  { GDK_Home,    "[1~",  "[1~",   "[H" },
  { GDK_End,     "[7~",  "[7~",   "[Y" },
  { GDK_Prior,   "[5~",  "[5~",   "[V" },
  { GDK_Next,    "[6~",  "[6~",   "[U" },
  { GDK_Insert,  "[2~",  "[2~",  "[@"  },
  { GDK_Delete,  "[3~", "[3~",  "\177" },
  { 0,            NULL,   NULL,   NULL }
};

gint
ansi_term_key_press(GtkWidget *w, GdkEventKey *ev)
{
	int i, n;
	char buf[256];
	GList *list;
	KeyMap *keymap;
	int key;

	AnsiTerm *term;

	term = ANSI_TERM(w);

	/* check gau is in scroll mode */
	if (term->real_visible_line_begin != term->visible_line_begin) {
		scroll_mode_key_press(term, ev);
		return TRUE;
	}


	list = term->keymap_list;
	key = ev->keyval;
	if (key >= GDK_Shift_L && key <= GDK_Alt_R) return FALSE;
	/* check if this event is mapped */
	while (list) {
		keymap = (KeyMap *)(list->data);
		if (key == keymap->keyval) {
			if (ev->state & keymap->state) {
#if HANGUL
				if (hangul_input_state_get()) {
					flush_hangul(term);
					hangul_input_clear();
				}
#endif
				(*(keymap->func))(keymap->data);
				return TRUE;
			}
		}
		list = list->next;
	}
	// translate keypad input to normal input;
	if (key >= GDK_KP_Space && key < GDK_KP_0) {
		switch(key) { 
			case GDK_KP_Enter:
				key= GDK_Return;
				break;
			case GDK_KP_Home:
				key = GDK_Home;
				break;
			case GDK_KP_Left:
				key = GDK_Left;
				break;
			case GDK_KP_Up:
				key = GDK_Up;
				break;
			case GDK_KP_Right:
				key = GDK_Right;
				break;
			case GDK_KP_Down:
				key = GDK_Down;
				break;
			case GDK_KP_Page_Up:
				key = GDK_Page_Up;
				break;
			case GDK_KP_Page_Down:
				key = GDK_Page_Down;
				break;
			case GDK_KP_End:
				key = GDK_End;
				break;
			case GDK_KP_Insert:
				key = GDK_Insert;
				break;
			case GDK_KP_Delete:
				key = GDK_Delete;
				break;
			case GDK_KP_Multiply:
				key = '*';
				break;
			case GDK_KP_Subtract:
				key = '-';
				break;
			case GDK_KP_Divide:
				key = '/';
				break;
			case GDK_KP_Add:
				key = '+';
				break;
		}
	}
#if HANGUL
	if (hangul_input_state_get()) {
		if (term->vi_esc_hack && key == GDK_Escape) {
			flush_hangul(term);
			hangul_input_toggle(NULL);
			goto english;
		}
		if (ev->state & (GDK_CONTROL_MASK | GDK_MOD1_MASK)) {
			if (key <= 0xff && key >= 0x20) {
				flush_hangul(term);
				hangul_input_clear();
				goto english;
			}
		}
		if (key != GDK_BackSpace && (key < 0x20 && key >= 0xff)) {
			flush_hangul(term);
			hangul_input_clear();
			goto english;
		}
		/* ya sane hangul input */ 
		n = hangul_input_process(buf, &key);
		switch(n) {
			case AUTOMATA_SPECIAL:
				flush_hangul(term);
				goto english;
				break;
			case AUTOMATA_NEW:
				push_hangul(term, buf);
				break;
			case AUTOMATA_CORRECT:
				correct_hangul(term, buf);
				ansi_term_show_temp_hangul(term);
				break;
			case AUTOMATA_CORRECT_NEW:
				correct_hangul(term, buf);
				flush_hangul(term);
				push_hangul(term, buf + 2);
				break;
			case AUTOMATA_NULL:
				term->temp_hangul[0] = 0;
				ansi_term_expose_line(term, term->cu_y, term->cu_x, term->cu_x + 1);
				ansi_term_show_cursor(term);
				break;
			case AUTOMATA_ERROR:
				gdk_beep();
				break;
			default:
				g_print("error in %s\n", __FUNCTION__);
				break;
		}

		return TRUE;
	}

	english:
#endif
	if (key >= GDK_KP_0 && key <= GDK_KP_9) {
		key = key - GDK_KP_0 + '0';
	}
	if ((key >= 0x20) && (key <= 0xff)) {
		if (ev->state & GDK_CONTROL_MASK) {
			if (key >= 'A' && key <= 'Z') 
				key -= 'A' -'a';
			if ((key >= 'a') && (key <= 'z') ) {
				key -= 'h' - '\b';
				buf[0] = key;
				term->from_term(buf, 1);
				return TRUE;
			} else if (key == ']' || key == '\\' || key == '[') {
				buf[0]= key & 0x1f;
				term->from_term(buf, 1);
				return TRUE;
			}
				return FALSE;
		} 
		if (ev->state & (GDK_MOD1_MASK | GDK_MOD3_MASK)) {
			buf[0] = ESC;
			buf[1] = key;
			term->from_term(buf, 2);
			return TRUE;
		}
		buf[0] = key;
		term->from_term( buf, 1);
	} else if (key == GDK_Return) {
		if (crlf == 0) {
			buf[0] = '\r';
			term->from_term( buf, 1);
		} else if (crlf == 2) {
			buf[0] = '\n';
			term->from_term( buf, 1);
		} else if (crlf == 1) {
			buf[0] = '\r';
			buf[1] = '\n';
			term->from_term (buf, 2);
		}
	} else if (key == GDK_BackSpace) {
		buf[0] = '\b';
		term->from_term(buf, 1);
	} else if (key == GDK_Tab) {
		buf[0] = '\t';
		term->from_term(buf, 1);
	} else if (key == GDK_Escape) {
		buf[0] = ESC;
		term->from_term(buf, 1);
	} else {
		for (i = 0; vt_keys[i].code; i++) {
			if (vt_keys[i].code == key) {
				if(term->mode & CUR_APL) g_snprintf(buf, 256, "%c%s", ESC, vt_keys[i].vt100_app);
				else g_snprintf(buf, 256, "%c%s", ESC, vt_keys[i].linux_con);
				term->from_term( buf, strlen(buf));
				break;
			}
		}
	}
	return TRUE; /* eat event */
}

#if HANGUL
void 
ansi_term_show_temp_hangul(AnsiTerm *term)
{
	if(term->temp_hangul[0]) {
		gdk_draw_rectangle(GTK_WIDGET(term)->window, term->black_gc, TRUE,
			term->cu_x *term->font_width, term->cu_y * term->font_height,
			term->font_width*2, term->font_height);
		gdk_draw_text(GTK_WIDGET(term)->window, term->font, term->white_gc, 
			term->cu_x * term->font_width, 
			term->cu_y * term->font_height + term->font_ascent,
			term->temp_hangul, 2);
	}
}

void
flush_hangul(AnsiTerm *term)
{
	if (term->temp_hangul[0]) {
		//ansi_term_hide_cursor(term);
		term->from_term(term->temp_hangul, 2);
		term->temp_hangul[0] = 0;
	}
}

void
push_hangul(AnsiTerm *term, char *buf)
{
	if (term->temp_hangul[0]) {
		term->from_term(term->temp_hangul, 2);
	}
	term->temp_hangul[0] = buf[0];
	term->temp_hangul[1] = buf[1];
	ansi_term_show_temp_hangul(term);
}

void
correct_hangul(AnsiTerm *term, char *buf)
{
	term->temp_hangul[0] = *buf++;
	term->temp_hangul[1] = *buf;
}
#endif

void
scroll_mode_page_up(AnsiTerm *term)
{
    if (((int)term->adjustment->value) == 0) return;
    term->adjustment->value -= term->row;
    if (term->adjustment->value < 0) term->adjustment->value = 0;
    gtk_signal_emit_by_name(GTK_OBJECT(term->adjustment), "value_changed");
}

void
scroll_mode_end(AnsiTerm *term)
{
	term->adjustment->value = term->adjustment->upper - term->row;
	gtk_signal_emit_by_name(GTK_OBJECT(term->adjustment), "value_changed");
}

void
scroll_mode_page_down(AnsiTerm *term)
{
    if (term->adjustment->value == (term->adjustment->upper - term->row)) return;
    term->adjustment->value += term->row;
    if (term->adjustment->value > (term->adjustment->upper - term->row)) 
	    term->adjustment->value = term->adjustment->upper - term->row;
    gtk_signal_emit_by_name(GTK_OBJECT(term->adjustment), "value_changed");
}

void
scroll_mode_line_up(AnsiTerm *term)
{
    if (((int)term->adjustment->value) == 0) return;
    term->adjustment->value--;
    if (term->adjustment->value < 0) term->adjustment->value = 0;
    gtk_signal_emit_by_name(GTK_OBJECT(term->adjustment), "value_changed");
}

void
scroll_mode_line_down(AnsiTerm *term)
{
    if (term->adjustment->value == (term->adjustment->upper - term->row)) return;
    term->adjustment->value++;
    if (term->adjustment->value > (term->adjustment->upper - term->row)) 
	term->adjustment->value = term->adjustment->upper - term->row;
    gtk_signal_emit_by_name(GTK_OBJECT(term->adjustment), "value_changed");
}

void
scroll_mode_key_press(AnsiTerm *term, GdkEventKey *ev)
{
    switch(ev->keyval) {
	case GDK_Up: case GDK_KP_Up: scroll_mode_line_up(term); break;
	case GDK_Down: case GDK_KP_Down: scroll_mode_line_down(term); break;
	case GDK_Page_Up: case GDK_KP_Page_Up: scroll_mode_page_up(term); break;
	case GDK_Page_Down: case GDK_KP_Page_Down: scroll_mode_page_down(term); break;
	case GDK_space: case GDK_Return: case GDK_Escape: scroll_mode_end(term); break;
    }
}

int
scrollbar_move_callback(GtkAdjustment *adj, AnsiTerm *term)
{
    int new_visible_line_begin;
    
    if (term->adjustment->upper <= 0) return TRUE;

    new_visible_line_begin =
	((int)term->adjustment->value + term->saved_lines + 
	term->real_visible_line_begin - 
	(int)term->adjustment->upper + term->row)%term->saved_lines;
    if (term->visible_line_begin == new_visible_line_begin) return TRUE;
    term->visible_line_begin = new_visible_line_begin;
    if (term->real_visible_line_begin == term->visible_line_begin &&
	term->tmp_len) {
	ansi_term_flush(term);
    }
    ansi_term_queue_draw(term);
    return TRUE;
}

==> gau-0.4.1.tar.gz: gau-0.4.1/ansiinput.c <==
/* this file is a part of gau software, (C) Hwang chi-deok 1997, 1998, 1999    */

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include "ansiterm.h"
#include "gau.h"


static void scroll_mode_key_press(AnsiTerm *term, GdkEventKey *ev);
int crlf = 0;
static void scroll_mode_page_up(AnsiTerm *);
static void scroll_mode_page_down(AnsiTerm *);

static void
scroll_up(AnsiTerm *term)
{
	scroll_mode_page_up(term);
}

void
keymap_init(AnsiTerm *term)
{
	KeyMap *keymap;
	keymap = g_new(KeyMap, 1);
	keymap->keyval = GDK_Page_Up;
	keymap->state = GDK_SHIFT_MASK;
	keymap->func = (KeyMapFunc)scroll_up;
	keymap->data = term;
	term->keymap_list = g_list_append(term->keymap_list, keymap);
}

/* Structure to hold escape sequences. */
struct escseq {
  int code;
  char *linux_con;
  char *vt100_app;
  char *ansi;
};

/* Escape sequences for different terminal types. */
static struct escseq vt_keys[] = {
  { GDK_F1,      "[[A", "[11~", "OP" },
  { GDK_F2,      "[[B", "[12~", "OQ" },
  { GDK_F3,      "[[C", "[13~", "OR" },
  { GDK_F4,      "[[D", "[14~", "OS" },
  { GDK_F5,      "[[E", "[15~", "OT" },
  { GDK_F6,      "[17~", "[17~", "OU" },
  { GDK_F7,      "[18~", "[18~", "OV" },
  { GDK_F8,      "[19~", "[19~", "OW" },
  { GDK_F9,      "[20~", "[20~", "OX" },
  { GDK_F10,     "[21~", "[21~", "OY" },
  { GDK_F11,     "[23~", "[23~", "OZ" },
  { GDK_F12,     "[24~", "[24~", "OA" },
  { GDK_Up,      "[A",   "OA",   "[A" },
  { GDK_Left,    "[D",   "OD",   "[D" },
  { GDK_Right,   "[C",   "OC",   "[C" },
  { GDK_Down,    "[B",   "OB",   "[B" },
  { GDK_Home,    "[1~",  "[1~",   "[H" },
  { GDK_End,     "[7~",  "[7~",   "[Y" },
  { GDK_Prior,   "[5~",  "[5~",   "[V" },
  { GDK_Next,    "[6~",  "[6~",   "[U" },
  { GDK_Insert,  "[2~",  "[2~",  "[@"  },
  { GDK_Delete,  "[3~", "[3~",  "\177" },
  { 0,            NULL,   NULL,   NULL }
};

gint
ansi_term_key_press(GtkWidget *w, GdkEventKey *ev)
{
	int i;
	char buf[256];
	GList *list;
	KeyMap *keymap;
	int key;

	AnsiTerm *term;

	if (schat && schat->visible) {
	    gtk_widget_event (GTK_WIDGET(schat->chat), (GdkEvent *)ev);
	    gtk_widget_grab_focus (GTK_WIDGET(schat->chat));
	    return TRUE;
	}

	term = ANSI_TERM(w);

	/* check gau is in scroll mode */
	if (term->real_visible_line_begin != term->visible_line_begin) {
		scroll_mode_key_press(term, ev);
		return TRUE;
	}


	list = term->keymap_list;
	key = ev->keyval;
	if (key >= GDK_Shift_L && key <= GDK_Alt_R) return FALSE;
	/* check if this event is mapped */
	while (list) {
		keymap = (KeyMap *)(list->data);
		if (key == keymap->keyval) {
			if (ev->state & keymap->state) {
				(*(keymap->func))(keymap->data);
				return TRUE;
			}
		}
		list = list->next;
	}
	if ((ev->state & (GDK_MOD1_MASK | GDK_MOD4_MASK)) && (ev->length == 1)) {
		buf[0] = ESC;
		buf[1] = ev->string[0];
		term->from_term(buf, 2);
		return TRUE;
	} else if (key == GDK_Return) {
		if (crlf == 0) {
			buf[0] = '\r';
			term->from_term( buf, 1);
		} else if (crlf == 2) {
			buf[0] = '\n';
			term->from_term( buf, 1);
		} else if (crlf == 1) {
			buf[0] = '\r';
			buf[1] = '\n';
			term->from_term (buf, 2);
		}
	} else if (key == GDK_BackSpace) {
		buf[0] = '\b';
		term->from_term(buf, 1);
	} else if (key == GDK_Tab) {
		buf[0] = '\t';
		term->from_term(buf, 1);
	} else if (key == GDK_Escape) {
		buf[0] = ESC;
		term->from_term(buf, 1);
	} else if (ev->length > 0) {
		term->from_term(ev->string, ev->length);
	} else {
		for (i = 0; vt_keys[i].code; i++) {
			if (vt_keys[i].code == key) {
				if(term->mode & CUR_APL) g_snprintf(buf, sizeof(buf), "%c%s", ESC, vt_keys[i].vt100_app);
				else g_snprintf(buf, sizeof(buf), "%c%s", ESC, vt_keys[i].linux_con);
				term->from_term( buf, strlen(buf));
				break;
			}
		}
		if (!vt_keys[i].code) return FALSE;
	}
	return TRUE; /* eat event */
}

static void
scroll_mode_page_up(AnsiTerm *term)
{
    if (((int)term->adjustment->value) == 0) return;
    term->adjustment->value -= term->row;
    if (term->adjustment->value < 0) term->adjustment->value = 0;
    gtk_signal_emit_by_name(GTK_OBJECT(term->adjustment), "value_changed");
}

static void
scroll_mode_end(AnsiTerm *term)
{
	term->adjustment->value = term->adjustment->upper - term->row;
	gtk_signal_emit_by_name(GTK_OBJECT(term->adjustment), "value_changed");
}

static void
scroll_mode_page_down(AnsiTerm *term)
{
    if (term->adjustment->value == (term->adjustment->upper - term->row)) return;
    term->adjustment->value += term->row;
    if (term->adjustment->value > (term->adjustment->upper - term->row)) 
	    term->adjustment->value = term->adjustment->upper - term->row;
    gtk_signal_emit_by_name(GTK_OBJECT(term->adjustment), "value_changed");
}

static void
scroll_mode_line_up(AnsiTerm *term)
{
    if (((int)term->adjustment->value) == 0) return;
    term->adjustment->value--;
    if (term->adjustment->value < 0) term->adjustment->value = 0;
    gtk_signal_emit_by_name(GTK_OBJECT(term->adjustment), "value_changed");
}

static void
scroll_mode_line_down(AnsiTerm *term)
{
    if (term->adjustment->value == (term->adjustment->upper - term->row)) return;
    term->adjustment->value++;
    if (term->adjustment->value > (term->adjustment->upper - term->row)) 
	term->adjustment->value = term->adjustment->upper - term->row;
    gtk_signal_emit_by_name(GTK_OBJECT(term->adjustment), "value_changed");
}

static void
scroll_mode_key_press(AnsiTerm *term, GdkEventKey *ev)
{
    switch(ev->keyval) {
	case GDK_Up: case GDK_KP_Up: scroll_mode_line_up(term); break;
	case GDK_Down: case GDK_KP_Down: scroll_mode_line_down(term); break;
	case GDK_Page_Up: case GDK_KP_Page_Up: scroll_mode_page_up(term); break;
	case GDK_Page_Down: case GDK_KP_Page_Down: scroll_mode_page_down(term); break;
	case GDK_space: case GDK_Return: case GDK_Escape: scroll_mode_end(term); break;
    }
}
